"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const handleOptions_1 = require("./handleOptions");
function injectExtraAPI(ctx) {
    const { layoutComponentMap } = ctx.themeAPI;
    /**
     * A function used to check whether layout exists
     */
    const isLayoutExists = (name) => layoutComponentMap[name] !== undefined;
    /**
     * Get layout
     */
    ctx.getLayout = (name, fallback) => {
        return isLayoutExists(name) ? name : fallback || "Layout";
    };
}
module.exports = (options, ctx) => {
    injectExtraAPI(ctx);
    const { pageEnhancers, indexPages } = handleOptions_1.handleOptions(options, ctx);
    return {
        name: "vuepress-plugin-hub",
        /**
         * 1. Execute `pageEnhancers` generated in handleOptions
         */
        extendPageData(pageCtx) {
            const { frontmatter: rawFrontMatter } = pageCtx;
            pageEnhancers.forEach(({ when, data = {}, frontmatter = {} }) => {
                if (when(pageCtx)) {
                    Object.keys(frontmatter).forEach(key => {
                        /**
                         * Respect the original frontmatter in markdown.
                         */
                        if (!rawFrontMatter[key]) {
                            rawFrontMatter[key] = frontmatter[key];
                        }
                    });
                    Object.assign(pageCtx, data);
                }
            });
        },
        /**
         * 2. Create pages according to user's config.
         */
        async ready() {
            const { pages } = ctx;
            /**
             * 2.1 Add paths of indexed pages to index pages.
             */
            for (let { permalink, frontmatter } of indexPages) {
                frontmatter.indexed = pages.filter(({ regularPath }) => handleOptions_1.isIndexed(regularPath, permalink, frontmatter.subdirlevel));
            }
            /**
             * 2.2 Combine all pages.
             *
             *   - Index pages.
             */
            const allExtraPages = [...indexPages];
            await Promise.all(allExtraPages.map(async (page) => ctx.addPage(page)));
        }
    };
};
